/*
 * Sixth 3D engine. Copyright ©2012-2018, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 *
 */

package eu.svjatoslav.sixth.e3d.gui;

import eu.svjatoslav.sixth.e3d.gui.humaninput.MouseClick;
import eu.svjatoslav.sixth.e3d.gui.humaninput.MouseInteractionController;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferByte;
import java.awt.image.WritableRaster;

public class RenderingContext {

    public static final int bufferedImageType = BufferedImage.TYPE_4BYTE_ABGR;
    public final Graphics2D graphics;
    public final byte[] pixels;
    public final int width;
    public final int height;
    public final int xCenter;
    public final int yCenter;
    public final double zoom;
    final BufferedImage bufferedImage;
    public int frameNumber = 0;

    /**
     * Used to signal that actual rendering should be performed. It is useful to
     * skip rendering when we only want to detect mouse clicks intersections
     * without actually updating rendered frame.
     */
    public boolean doRender = true; // TODO: make use of the variable

    /**
     * Mouse click. During rendering we can detect which item user clicked on.
     */
    public MouseClick mouseClick;

    /**
     * Item that user clicked on.
     */
    public MouseInteractionController clickedItem;

    public RenderingContext(final int width, final int height) {
        this.width = width;
        this.height = height;

        this.xCenter = width / 2;
        this.yCenter = height / 2;

        this.zoom = width / 3d;

        bufferedImage = new BufferedImage(width, height, bufferedImageType);

        final WritableRaster raster = bufferedImage.getRaster();
        final DataBufferByte dbi = (DataBufferByte) raster.getDataBuffer();
        pixels = dbi.getData();

        graphics = (Graphics2D) bufferedImage.getGraphics();
        graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        graphics.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
    }

}
