/*
 * Sixth 3D engine. Copyright ©2012-2018, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 *
 */

package eu.svjatoslav.sixth.e3d.renderer.raster.shapes.basic.texturedpolygon;

import eu.svjatoslav.sixth.e3d.geometry.Point2D;

public class PolygonBorderInterpolator implements
        Comparable<PolygonBorderInterpolator> {

    // on-screen coordinates
    Point2D p1;
    Point2D p2;
    double distanceFromY1;
    private int height;
    private int width;
    private int absoluteHeight;
    private double twidth;
    private double theight;
    // texture coordinates
    private Point2D tp1;
    private Point2D tp2;


    @Override
    public boolean equals(final Object o) {
        if (o == null) return false;

        return o instanceof PolygonBorderInterpolator && compareTo((PolygonBorderInterpolator) o) == 0;
    }

    @Override
    public int hashCode() {
        int result = width;
        result = 31 * result + absoluteHeight;
        return result;
    }

    @Override
    public int compareTo(final PolygonBorderInterpolator o) {
        if (absoluteHeight < o.absoluteHeight)
            return 1;
        if (absoluteHeight > o.absoluteHeight)
            return -1;

        if (width < o.width)
            return 1;
        if (width > o.width)
            return -1;

        return 0;
    }

    public boolean containsY(final int y) {

        if (p1.y < p2.y) {
            if (y >= p1.y)
                if (y <= p2.y)
                    return true;
        } else if (y >= p2.y)
            if (y <= p1.y)
                return true;

        return false;
    }

    public double getTX() {

        if (height == 0)
            return (tp2.x + tp1.x) / 2d;

        return tp1.x + ((twidth * distanceFromY1) / height);
    }

    public double getTY() {

        if (height == 0)
            return (tp2.y + tp1.y) / 2d;

        return tp1.y + ((theight * distanceFromY1) / height);
    }

    public int getX() {

        if (height == 0)
            return (int) ((p2.x + p1.x) / 2d);

        return (int) (p1.x + ((width * distanceFromY1) / height));
    }

    public void setCurrentY(final int y) {
        distanceFromY1 = y - p1.y;
    }

    public void setPoints(final Point2D p1, final Point2D p2,
                          final Point2D tp1, final Point2D tp2) {

        this.p1 = p1;
        this.p2 = p2;
        this.tp1 = tp1;
        this.tp2 = tp2;

        height = (int) (p2.y - p1.y);
        width = (int) (p2.x - p1.x);
        absoluteHeight = Math.abs(height);

        twidth = tp2.x - tp1.x;
        theight = tp2.y - tp1.y;
    }

}
