/*
 * Sixth 3D engine. Copyright ©2012-2018, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 *
 */

package eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite;

import eu.svjatoslav.sixth.e3d.geometry.Point2D;
import eu.svjatoslav.sixth.e3d.geometry.Point3D;
import eu.svjatoslav.sixth.e3d.math.Transform;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.basic.texturedpolygon.TexturedPolygon;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite.base.AbstractCompositeShape;
import eu.svjatoslav.sixth.e3d.renderer.raster.texture.Texture;

public class TexturedRectangle extends AbstractCompositeShape {

    public Point3D topLeft;
    public Point3D topRight;
    public Point3D bottomRight;
    public Point3D bottomLeft;
    public Point2D textureTopLeft;
    public Point2D textureTopRight;
    public Point2D textureBottomRight;
    public Point2D textureBottomLeft;
    private Texture texture;

    public TexturedRectangle(final Transform transform) {
        super(transform);
    }

    public TexturedRectangle(final Transform transform, final int width,
                             final int height, final int maxTextureUpscale) {
        this(transform, width, height, width, height, maxTextureUpscale);
    }

    public TexturedRectangle(final Transform transform, final int width,
                             final int height, final int textureWidth, final int textureHeight,
                             final int maxTextureUpscale) {

        super(transform);

        initialize(width, height, textureWidth, textureHeight,
                maxTextureUpscale);
    }

    public Texture getTexture() {
        return texture;
    }

    public void initialize(final double width, final double height,
                           final int textureWidth, final int textureHeight,
                           final int maxTextureUpscale) {

        topLeft = new Point3D(-width / 2, -height / 2, 0);
        topRight = new Point3D(width / 2, -height / 2, 0);
        bottomRight = new Point3D(width / 2, height / 2, 0);
        bottomLeft = new Point3D(-width / 2, height / 2, 0);

        texture = new Texture(textureWidth, textureHeight, maxTextureUpscale);

        textureTopRight = new Point2D(textureWidth, 0);
        textureTopLeft = new Point2D(0, 0);
        textureBottomRight = new Point2D(textureWidth, textureHeight);
        textureBottomLeft = new Point2D(0, textureHeight);

        final TexturedPolygon texturedPolygon1 = new TexturedPolygon(topLeft,
                topRight, bottomRight, textureTopLeft, textureTopRight,
                textureBottomRight, texture);

        texturedPolygon1
                .setMouseInteractionController(mouseInteractionController);

        final TexturedPolygon texturedPolygon2 = new TexturedPolygon(topLeft,
                bottomLeft, bottomRight, textureTopLeft, textureBottomLeft,
                textureBottomRight, texture);

        texturedPolygon2
                .setMouseInteractionController(mouseInteractionController);

        addShape(texturedPolygon1);
        addShape(texturedPolygon2);
    }

    public void initialize(final int width, final int height,
                           final int maxTextureUpscale) {
        initialize(width, height, width, height, maxTextureUpscale);
    }

}
